import pandas as pd

def generate_diagnostics(df: pd.DataFrame) -> pd.DataFrame:
    """
    Generate a DataFrame with diagnostics to understand how the standardization process impacts the raw data. 

    This function groups the output data by the standardized plaintiff name and aggregates the raw values into a string
    to make it easier to visually inspect the output. This method also adds a count of the number of raw values that were standardized 
    to the same value. 

    """
    count_raw_names_by_clean_name = df.groupby('first_plaintiff_clean').agg(
        count_first_plaintiff = ('first_plaintiff', 'nunique'),
        values_first_plaintiff = ('first_plaintiff', lambda x: ','.join(x))
        ).reset_index()

    return count_raw_names_by_clean_name



from loguru import logger
import pandas as pd


from cleancourt.check_spaces import *
from cleancourt.clean_data import *
from cleancourt.cosine_similarity import *
from cleancourt.integrate_output import *
from cleancourt.parse_names import *
from cleancourt.separate_parent_company import separate_company_names


def full_clean_names(messy_names):
    
    df = pd.DataFrame()

    df['clean_names_1'] = [clean_data(x) for x in tqdm(messy_names)]



    # Parse the people names and format
    logger.info("PARSING PEOPLE NAMES")
    duplicates_dict_people = format_people_names(df['clean_names_1'])

    # Merge names into df
    logger.info("MERGING PEOPLE NAMES")
    df['formatted_people_names'] = df.apply(lambda x: map_values(x.clean_names_1, duplicates_dict_people), axis=1)

    # Parse company names
    logger.info("PARSING COMPANY NAMES")
    duplicates_dict_company = link_company_names(df['clean_names_1'])

    # Merge company names into df
    logger.info("MERGING COMPANY NAMES")
    df['formatted_company_names'] = df.apply(lambda x: map_values(x.clean_names_1, duplicates_dict_company), axis=1)

    # Create column of all company and people names
    logger.info("ADDING ALL NAMES TO DF")
    df['clean_names_2'] = df.apply(lambda x: combine_columns(x.clean_names_1, x.formatted_people_names, x.formatted_company_names), axis=1)

    # Check for erroneous spaces that may prevent names from being matched
    logger.info("CHECKING SPACES")
    duplicates_dict_spaces = check_spaces(df['clean_names_2'])

    # add values returned from no spaces to df
    logger.info("ADDING NO SPACES NAMES TO DF")
    df['clean_names_3'] = df.apply(lambda x: map_values(x.clean_names_2, duplicates_dict_spaces), axis=1)

    # Combine all cleaned names columns into one column for use
    df['clean_name_final'] = df.apply(lambda x: combine_columns_no_space(x.clean_names_2, x.clean_names_3), axis=1)

    return(df['clean_name_final'])




# Same as the full clean names function, only this function separates the company names by DBA/TA names
# In future iterations, this method will replace full_clean_names
def full_clean_names1(messy_names):
    
    df = pd.DataFrame()

    df['clean_names_1'] = [clean_data(x) for x in tqdm(messy_names)]

    logger.info("SEPARATING MANAGING NAMES")
    df['nameA'], df['nameB'] = separate_company_names(df['clean_names_1'])



    # Parse the people names and format
    logger.info("PARSING PEOPLE NAMES")
    duplicates_dict_peopleA = format_people_names( df['nameA'])
    duplicates_dict_peopleB = format_people_names( df['nameB'])


    # Merge names into df
    logger.info("MERGING PEOPLE NAMES")
    df['formatted_people_namesA'] = df.apply(lambda x: map_values(x.nameA, duplicates_dict_peopleA), axis=1)
    df['formatted_people_namesB'] = df.apply(lambda x: map_values(x.nameB, duplicates_dict_peopleB), axis=1)



    # Parse company names
    logger.info("PARSING COMPANY NAMES")
    duplicates_dict_companyA = link_company_names(df['nameA'])
    duplicates_dict_companyB = link_company_names(df['nameB'])

    # Merge company names into df
    logger.info("MERGING COMPANY NAMES")
    df['formatted_company_namesA'] = df.apply(lambda x: map_values(x.nameA, duplicates_dict_companyA), axis=1)
    df['formatted_company_namesB'] = df.apply(lambda x: map_values(x.nameB, duplicates_dict_companyB), axis=1)

    # Create column of all company and people names
    logger.info("ADDING ALL NAMES TO DF")
    df['clean_names_2a'] = df.apply(lambda x: combine_columns(x.nameA, x.formatted_people_namesA, x.formatted_company_namesA), axis=1)
    df['clean_names_2b'] = df.apply(lambda x: combine_columns(x.nameB, x.formatted_people_namesB, x.formatted_company_namesB), axis=1)

    # Check for erroneous spaces that may prevent names from being matched
    logger.info("CHECKING SPACES")
    duplicates_dict_spacesA = check_spaces(df['clean_names_2a'])
    duplicates_dict_spacesB = check_spaces(df['clean_names_2b'])

    # add values returned from no spaces to df
    logger.info("ADDING NO SPACES NAMES TO DF")
    df['clean_names_3a'] = df.apply(lambda x: map_values(x.clean_names_2a, duplicates_dict_spacesA), axis=1)
    df['clean_names_3b'] = df.apply(lambda x: map_values(x.clean_names_2b, duplicates_dict_spacesB), axis=1)

    # Combine all cleaned names columns into one column for use
    df['clean_name_finalA'] = df.apply(lambda x: combine_columns_no_space(x.clean_names_2a, x.clean_names_3a), axis=1)
    df['clean_name_finalB'] = df.apply(lambda x: combine_columns_no_space(x.clean_names_2b, x.clean_names_3b), axis=1)

    return(df['clean_name_finalA'], df['clean_name_finalB'])


import re
from tqdm import tqdm


def remove_company_connector(string):
    string = re.sub("(.+) as a? ?management agent for (.+)", "\\2 /-/ \\1", string) # Remove any appareance of as management agent for
    string = re.sub("(.+) successor in interest to (.+)", "\\2 /-/ \\1", string)
    string = re.sub("(.+) [as ]?assignee of (.+)", "\\2 /-/ \\1", string)
    string = re.sub("(.+) [as ]?managing agent for (.+)", "\\2 /-/ \\1", string)
    string = re.sub("(.+) [as ]?m/a for (.+)", "\\2 /-/ \\1", string)
    string = re.sub("(.+) by (.+) as managing agent$", "\\1 /-/ \\2", string)
    string = re.sub("(.+) a[^A-z]?s[^A-z]?o[^A-z]? (.+)", "\\2 /-/ \\1", string) # Remove any appearance of a.s.o. and what follows
    string = re.sub("(.+) d[^A-z]?b[^A-z]?a[^A-z]? (.+)", "\\2 /-/ \\1", string) # Remove any appearance of d.b.a and what follows
    string = re.sub("(.+) as agent for (.+)", "\\2 /-/ \\1", string)
    string = re.sub("(.+) agent for (.+)", "\\2 /-/ \\1", string)
    string = re.sub("(.+) as mg agent for (.+)", "\\2 /-/ \\1", string)
    string = re.sub("(.+) mg agt (.+)", "\\2 /-/ \\1", string)
    string = re.sub("(.+) as managing for (.+)", "\\2 /-/ \\1", string)
    string = re.sub("(.+) asset\.? management for (.+)", "\\2 /-/ \\1", string)
    string = re.sub("(.+) doing business as (.+)", "\\2 /-/ \\1", string)

    return string


def separate_company_names(names):
    plaintiffs = []
    plaintiff_parents = []
    
    for name in tqdm(names):
        split_names = re.split(" /-/ ", remove_company_connector(name))

        plaintiff_parents.append(split_names[1])  if(len(split_names) == 2)  else  plaintiff_parents.append('') 
        plaintiffs.append(split_names[0])

    return(plaintiffs, plaintiff_parents)






from loguru import logger
import probablepeople as pp
from tqdm import tqdm
import pandas as pd
import json
import re
from cleancourt.filter_types import filter_people


def format_people_names(df):

    '''Take in a list or df of one column and return a dictionary with each name in First Last format'''

    # Group data by exact matches and count the number of values
    logger.info("GROUPING DATA")
    grouped_df = df.value_counts().rename_axis('first_plaintiff').reset_index(name='counts')

    # Grab the names of each plaintiff and place into a list
    # Note: Order of this list does matter. 
    # Names are prioritized based on the number of exact matches in the document

    names = grouped_df.first_plaintiff
    
    names_dict = {}

    # Iterate over names

    for name in tqdm(names):
        
        # Check if name is a person
        is_person = filter_people(name)
        
        # If is a person, grab name components from parsed names and place in a common format
        if(is_person):
            val = pp.tag(name)
            try:
                firstI = val[0]['FirstInitial']
            except KeyError:
                firstI = ""

            try:
                first = val[0]['GivenName']
            except KeyError:
                first = ""
            
            try:
                middle = val[0]['MiddleName']
            except KeyError:
                middle = ""
            
            try:
                middleI = val[0]['MiddleInitial']
            except KeyError:
                middleI = ""

            try: 
                last = val[0]['Surname']
            except KeyError:
                last = ""

            try: 
                suffix = val[0]['SuffixGenerational']
            except KeyError:
                suffix = ""

            full_name = firstI + " " + first + " " + middle + " " + middleI +" " + last + " " + suffix

            names_dict[name] = re.sub(" +", " ", full_name.strip())

    return names_dict

